//#include "Arduino.h"  compiler freezes if this is included
#include <Wire.h>
// Arduino Uno pins ******************************************************************

//                              pin
const int pinSerial0RX       =  0; //   
const int pinSerial0TX       =  1; //  
const int pinInt0            =  2; //
const int pinInt1_PWM0       =  3; //
const int pinGeneric0        =  4; //

const int pinPWM1            =  5; //
const int pinPWM2            =  6; //
const int pinGeneric1        =  7; //
const int pinGeneric2        =  8; //
const int pinPWM3            =  9; //

const int pinSPISS_PWM4      = 10; // 
const int pinSPIMOSI_PWM5    = 11; //
const int pinSPIMISO         = 12; //
const int pinSPISCK_LED      = 13; //


// also 0 .. 5 for analogRead
// A4 = SDA
// A5 = SCL

// end Arduino Uno pins **************************************************************

#define MCP4726_CMD_WriteDAC  (0x40)  // Writes data to the DAC
#define Scale4725 4096
//#define debug 1

const int pinPeriod = 0;             // analog pin 0
const int pinFrozen = pinGeneric0;   // 4

const unsigned long msOneSecond    = 1000;
const        double pi             = 3.1415926;

class MCP4725
  { public:
      MCP4725();
      void begin(uint8_t a);  
      void SetVoltage( uint16_t output);
      void SetScaledVoltage(double Trig);
    private:
      int OldIntScaled;
      bool New;
      uint8_t _i2caddr;
  }; // MCP4725

MCP4725::MCP4725()
  {
  }

void MCP4725::begin(uint8_t addr)
  { _i2caddr = addr;
    New = true;
   } // MCP4725::begin
 
void MCP4725::SetVoltage(uint16_t output)
  { uint8_t twbrback = TWBR;
    TWBR = ((F_CPU / 400000L) - 16) / 2; // Set I2C frequency to 400kHz
    Wire.beginTransmission(_i2caddr);

    Wire.write(MCP4726_CMD_WriteDAC);
    Wire.write(output / 16);                   // Upper data bits          (D11.D10.D9.D8.D7.D6.D5.D4)
    Wire.write((output % 16) << 4);            // Lower data bits          (D3.D2.D1.D0.x.x.x.x)
    Wire.endTransmission();
    TWBR = twbrback;
  } // MCP4725::SetVoltage

void MCP4725::SetScaledVoltage(double Trig)
  { // range of Trig is [-1, 1], so scale by Scale4725/2
    
    int IntScaled;
    
    IntScaled = constrain((Trig + 1.0) * (Scale4725 / 2), 0, Scale4725-1);
    New = false;
    SetVoltage(IntScaled);
  } // MCP4725::SetScaledVoltage

// These addresses work with the DACs marked "P"; other markings indicate
// different addresses.  The addresses use the 7 high bits of the byte.  Although
// Bus Pirate right-pads the address to have them be 0xC0, 0xC2, and so on,
// wire accepts them as is, so 0xC4 becomes 0x62, 0xC6 becomes 0x63, etc.
//                    Bus Pirate            wire
//       marking    pin 6 low, high    pin 6 low, high 
//         AJnn       0xC0, 0xC2         0x60, 0x61
//         APnn       0xC4, 0xC6         0x62, 0x63
//         AQnn       0xC8, 0xCA         0x64, 0x6A
//         ARnn       OxCC, 0xCE         0x66, 0x67

#define SineDACaddr     0x63
#define CosineDACaddr   0x62
#define ScaleAnalogRead 1024

MCP4725 SineDAC;
MCP4725 CosineDAC;

const unsigned long msMinPeriod         = 100;
const unsigned long msMaxPeriod         = 20 * msOneSecond;
const int           MinPeriodPinReading =  25;
const int           MaxPeriodPinReading = 950;
const int           PeriodPinNoise      =  10;
      
unsigned long msNextStartPeriod;
          int OldPeriodPinReading = 0;

void setup(void)
  { Serial.begin(9600);
    Serial.println("Hello!");
    delay(1000);
    Wire.begin();

    pinMode(pinFrozen, OUTPUT);
    
    SineDAC.begin  (SineDACaddr  );
    CosineDAC.begin(CosineDACaddr);
    
    msNextStartPeriod = 0;
    } // setup
  
unsigned long msPeriod;
unsigned long msStartPeriod;

void loop(void)
  { // send sin(theta)+1 to DAC.  Range is 0 to 2, so scale by Scale4725/2
          boolean Frozen;
              int PWMSine;
    unsigned long msTimeNow;
            float PeriodFractionPast;
            float PhaseRadians;
            float TheCosine, TheSine;  // separate to minimize lag between DACs
               int PeriodPinReading;
    
    PeriodPinReading = analogRead(pinPeriod);
    Frozen = (PeriodPinReading < MinPeriodPinReading) || (PeriodPinReading > MaxPeriodPinReading);
    #ifdef debug
       Serial.print("Frozen ");
       Serial.println(Frozen);
    #endif
    digitalWrite(pinFrozen, Frozen);
    if (abs(PeriodPinReading - OldPeriodPinReading) > PeriodPinNoise)
      { // change Period
        msPeriod = msMinPeriod + ((msMaxPeriod - msMinPeriod) * PeriodPinReading)/ ScaleAnalogRead;
        OldPeriodPinReading = PeriodPinReading;
        Serial.print("period (ms) = ");
        Serial.println(msPeriod);
      } // change period

    msTimeNow = millis();
    if (msTimeNow >= msNextStartPeriod)
      { msStartPeriod = msTimeNow;
        msNextStartPeriod = msTimeNow + msPeriod;
      }
      
    PeriodFractionPast = float(msTimeNow - msStartPeriod) / float(msPeriod);
    PhaseRadians = 2.0 * pi * PeriodFractionPast;

    if (!Frozen)
      { TheSine   = sin(PhaseRadians);
        TheCosine = cos(PhaseRadians);
      }
    #ifdef debug
      PWMSine = (int) (127*(TheSine + 1));
      Serial.println(TheSine);
    #endif
    SineDAC.SetScaledVoltage  (TheSine  );
    CosineDAC.SetScaledVoltage(TheCosine);
   } // loop
